@@ -1,18 +1,15 @@
-#!/bin/bash
+#!/usr/bin/env bash
 
 set -e
 
 source $(dirname $0)/lib.sh
 
-req_env_var GOSRC OS_RELEASE_ID OS_RELEASE_VER SHORT_APTGET
-
-install_ooe
+req_env_vars GOSRC OS_RELEASE_ID OS_RELEASE_VER SHORT_APTGET TEST_DRIVER
 
 show_env_vars
 
 cd $GOSRC
-
-echo "Setting up $OS_RELEASE_ID $OS_RELEASE_VER"
+msg "Setting up $OS_RELEASE_ID $OS_RELEASE_VER"
 case "$OS_RELEASE_ID" in
     fedora)
         $LONG_DNFY update  # install latest packages
@@ -20,9 +17,17 @@ case "$OS_RELEASE_ID" in
             $SHORT_DNFY install $RPMS_REQUIRED
         [[ -z "$RPMS_CONFLICTING" ]] || \
             $SHORT_DNFY erase $RPMS_CONFLICTING
+        # Only works on Fedora VM images
+        bash "$SCRIPT_BASE/add_second_partition.sh"
+        if [[ "$TEST_DRIVER" == "devicemapper" ]]; then
+            $SHORT_DNFY install lvm2
+            devicemapper_setup
+        fi
         ;;
     ubuntu)
         $SHORT_APTGET update  # Fetch latest package metadata
+        [[ -z "$DEBS_HOLD" ]] || \
+            apt-mark hold $DEBS_HOLD
         $LONG_APTGET upgrade # install latest packages
         [[ -z "$DEBS_REQUIRED" ]] || \
             $SHORT_APTGET -q install $DEBS_REQUIRED
@@ -35,6 +40,4 @@ case "$OS_RELEASE_ID" in
 esac
 
 install_fuse_overlayfs_from_git
-
-echo "Installing common tooling"
-#make install.tools
+install_bats_from_git
